@@ -1,4 +1,4 @@
-from urllib.parse import quote
+import re
 import copy
 
 from mau.lexers.base_lexer import TokenTypes, Token, TokenError
@@ -26,9 +26,16 @@ from mau.parsers.nodes import (
 
 
 def header_anchor(text, level):
-    return "h{}-{}-{}".format(
-        level, quote(text.lower())[:20], str(id(text))[:8]
-    )  # pragma: no cover
+    # Everything lowercase
+    sanitised_text = text.lower()
+
+    # Get only letters, numbers, dashes, and spaces
+    sanitised_text = "".join(re.findall("[a-z0-9- ]+", sanitised_text))
+
+    # Remove multiple spaces
+    sanitised_text = "-".join(sanitised_text.split())
+
+    return sanitised_text
 
 
 class MainParser(BaseParser):
@@ -47,6 +54,10 @@ class MainParser(BaseParser):
         self._kwargs = {}
         self._title = None
 
+        self.header_anchor = self.variables.get(
+            "mau.header_anchor_function", header_anchor
+        )
+
     def _add_footnotes(self, footnotes):
         self.footnotes.extend(footnotes)
 
@@ -129,7 +140,7 @@ class MainParser(BaseParser):
 
         text = self.get_token(TokenTypes.TEXT).value
         level = len(header)
-        anchor = header_anchor(text, level)
+        anchor = self.header_anchor(text, level)
 
         if in_toc:
             self.headers.append((text, level, anchor))
@@ -235,16 +246,22 @@ class MainParser(BaseParser):
 
     def _parse_source_block(self, content, secondary_content, title, args, kwargs):
         delimiter = kwargs.pop("callouts", ":")
+        highlight_marker = kwargs.pop("highlight", "@")
 
-        # This removes callouts from the source code
-        # and maps line numbers to callouts and text
-        # {linenum:(callout,text)}
-        marked_lines = {}
+        # A dictionary that contains callout markers in
+        # the form {linenum:name}
+        callout_markers = {}
 
-        for linenum, line in enumerate(content):
-            if not line.endswith(delimiter):
-                continue
+        # A list of highlighted lines
+        highlighted_lines = []
 
+        lines_with_callouts = [
+            (linenum, line)
+            for linenum, line in enumerate(content)
+            if line.endswith(delimiter)
+        ]
+
+        for linenum, line in lines_with_callouts:
             # Remove the final delimiter
             line = line[:-1]
 
@@ -256,31 +273,36 @@ class MainParser(BaseParser):
             callout_name = splits[-1]
             line = delimiter.join(splits[:-1])
 
-            marked_lines[linenum] = (callout_name, line)
+            content[linenum] = line
 
-        # This maps callouts names to line numbers
-        # {callout_name:linenum}
-        callout_name_to_linenum = {}
-        for linenum, mark in marked_lines.items():
-            callout_name, fixed_line = mark
-            content[linenum] = fixed_line
-            callout_name_to_linenum[callout_name] = linenum
+            if callout_name == highlight_marker:
+                highlighted_lines.append(linenum)
+            else:
+                callout_markers[linenum] = callout_name
+
+        # A dictionary that contains the text for each
+        # marker in the form {name:text}
+        callout_contents = {}
 
-        # This reads the callout text and connects
-        # it with the text line
-        callouts = {}
         for line in secondary_content:
             if ":" not in line:
-                raise ParseError
+                raise ParseError(
+                    f"Callout description should be written as 'linuenum: text'. Missing ':' in '{line}'"
+                )
 
-            callout_name, text = line.split(": ")
+            name, text = line.split(":")
 
-            try:
-                linenum = callout_name_to_linenum[callout_name]
-            except KeyError:
-                raise ParseError(f"Callout {callout_name} can't be found")
+            if name not in callout_markers.values():
+                raise ParseError(
+                    f"Callout {name} has not been created in the source code"
+                )
 
-            callouts[linenum] = (callout_name, text)
+            text = text.strip()
+
+            callout_contents[name] = text
+
+        # Put markers and contents together
+        callouts = {"markers": callout_markers, "contents": callout_contents}
 
         # Source blocks must preserve the content literally
         textlines = [TextNode(line) for line in content]
@@ -293,6 +315,7 @@ class MainParser(BaseParser):
             SourceNode(
                 language,
                 callouts=callouts,
+                highlights=highlighted_lines,
                 delimiter=delimiter,
                 code=textlines,
                 title=title,
@@ -397,8 +420,7 @@ class MainParser(BaseParser):
         # Text should return a single sentence node
         result = p.nodes[0]
 
-        if len(p.footnotes) > 0:
-            self._add_footnotes(p.footnotes)
+        self._add_footnotes(p.footnotes)
 
         return result
 
